using System;
using System.Collections;
using System.Runtime.InteropServices;
using Visio = Microsoft.Office.Interop.Visio;

using NETVSL_Types;

namespace NETVSL_Base
{
  //----------------------------------
  internal abstract class TNETVSL_Addon { 
  //----------------------------------
    //--------------------------------
    // Basic data remembered by Addon.
    // Init in descendant constructor
    //--------------------------------
    protected TNETVSL_Base                    fOwnerVSL      ;
    protected String                          fName          ; 
    protected NETVSL_AddOnInfo_Attribs        fAttribs       ; 
    protected NETVSL_AddOnInfo_EnableMask     fEnableMask    ; 
    protected NETVSL_AddOnInfo_InvokeOnMask   fInvokeOnMask  ; 
    protected NETVSL_AddOnInfo_NotifyOnMask   fNotifyOnMask  ; 
    
    //--------------------------------
    // Read-only properties
    //--------------------------------
    public TNETVSL_Base                    OwnerVSL       { get { return fOwnerVSL     ; } }
    public String                          Name           { get { return fName         ; } }
    public NETVSL_AddOnInfo_Attribs        Attribs        { get { return fAttribs      ; } }
    public NETVSL_AddOnInfo_EnableMask     EnableMask     { get { return fEnableMask   ; } }
    public NETVSL_AddOnInfo_InvokeOnMask   InvokeOnMask   { get { return fInvokeOnMask ; } }
    public NETVSL_AddOnInfo_NotifyOnMask   NotifyOnMask   { get { return fNotifyOnMask ; } }
    
    //-----------------------
    public TNETVSL_Addon(
           TNETVSL_Base                    AOwnerVSL     ,
           String                          AName         , 
           NETVSL_AddOnInfo_Attribs        AAttribs      , 
           NETVSL_AddOnInfo_EnableMask     AEnableMask   , 
           NETVSL_AddOnInfo_InvokeOnMask   AInvokeOnMask , 
           NETVSL_AddOnInfo_NotifyOnMask   ANotifyOnMask )              
    {
      fOwnerVSL      =  AOwnerVSL      ;      
      fName          =  AName          ;
      fAttribs       =  AAttribs       ;
      fEnableMask    =  AEnableMask    ;
      fInvokeOnMask  =  AInvokeOnMask  ;
      fNotifyOnMask  =  ANotifyOnMask  ;      
      
      // in descendants, initialize the properties (Name etc)
    }
    
    //-------------------------------------
    public virtual NETVSL_ResultCodes Load() { 
    //-------------------------------------
      return NETVSL_ResultCodes.SUCCESS; 
    }
        
    //--------------------------------------------
    public virtual NETVSL_ResultCodes GetAddonInfo (     
    //--------------------------------------------
      Int32 SessionId , 
      ref String                          AName , 
      ref NETVSL_AddOnInfo_Attribs        AAttribs , 
      ref NETVSL_AddOnInfo_EnableMask     AEnableMask , 
      ref NETVSL_AddOnInfo_InvokeOnMask   AInvokeOnMask , 
      ref NETVSL_AddOnInfo_NotifyOnMask   ANotifyOnMask ) 
    {
      AName          = fName          ;
      AAttribs       = fAttribs       ;
      AEnableMask    = fEnableMask    ;
      AInvokeOnMask  = fInvokeOnMask  ;
      ANotifyOnMask  = fNotifyOnMask  ; 
      
      return  NETVSL_ResultCodes.SUCCESS;  
    }
    
    //----------------------------------------------
    public virtual NETVSL_ResultCodes IsAddOnEnabled (System.Int32 SessionId )  
    //----------------------------------------------
    {
      return NETVSL_ResultCodes.L2V_ENABLED;
    }

    //----------------------------------------
    public virtual void KillSession ( System.Int32 SessionId ) 
    //----------------------------------------
    {
    }

    //----------------------------------------
    public abstract NETVSL_ResultCodes Run (Int32   SessionId , String  Args ); 
    //----------------------------------------

    //----------------------------------------
    public virtual NETVSL_ResultCodes Unload (  ) 
    //----------------------------------------
    {
      return NETVSL_ResultCodes.SUCCESS;
    }    
  }   
	
	//=========================================
	[ClassInterface(ClassInterfaceType.None)] 
	public abstract class TNETVSL_Base: INETVSL { 
  //=========================================
    
    //..........................
    private class TTraceMessage {
    //..........................
      public NETVSL_AdapterMessage MsgType;
      public String                Msg;    
    }
    
    //-------------------------------------------------------
    // Data members
    //-------------------------------------------------------  
    protected ArrayList fAddons;	
    private   ArrayList fEndSessionIds;
    private   ArrayList fTraceMessages;  
    private   Visio.Application  fVisApp  = null;      
    
    //-------------------------------------------------------
    // Non-INETVSL methods (hence internal)
    //-------------------------------------------------------
    
    //----------------------------------------
    internal void Addon_Add(TNETVSL_Addon AAddon) {
    //----------------------------------------
      fAddons.Add(AAddon);
    }
    
    //----------------------------------------
    internal TNETVSL_Addon AddonFromIx1(int Ix1) {
    //----------------------------------------
      TNETVSL_Addon Result;
      Result = null;
      int Ix0 = Ix1 - 1;
      if ( (Ix0 >= 0) && (Ix0 < AddonCount) ) {
        Result = (TNETVSL_Addon) fAddons[Ix0];
      }
      return Result;
    }    
    
    //----------------------------------------
    internal void EnqueueEndSessionId (Int32 SessionId )   // 
    //----------------------------------------
    {
      fEndSessionIds.Add(SessionId);
    }    

    //----------------------------------------
    internal void EnqueueTraceMessage (NETVSL_AdapterMessage MsgType, String Msg)   // 
    //----------------------------------------
    {
      TTraceMessage ATM = new TTraceMessage();  
      ATM.MsgType       = MsgType;
      ATM.Msg           = Msg;
      fTraceMessages.Add(ATM);
    }   
    
    internal Visio.Application VisApp { get { return fVisApp ; } }
	  
    //----------------------------------------
    public TNETVSL_Base()
    //----------------------------------------
		{
		  //------------------------------
		  // Init our collections
		  //------------------------------
		  fAddons        = new ArrayList();
		  fEndSessionIds = new ArrayList();
      fTraceMessages = new ArrayList();   
      
		  //------------------------------
		  // Init our connection to Visio
		  //------------------------------           
      try {
        fVisApp = (Visio.Application) Marshal.GetActiveObject("visio.application");
      } catch {
      }  
      
      if (VisApp == null) {
        EnqueueTraceMessage(NETVSL_AdapterMessage.AddMessageWShow, "Couldn't connect to Visio instance");
      } else {
        EnqueueTraceMessage(NETVSL_AdapterMessage.AddMessage, "NETVSL connected to Visio instance"); 
      }      

		  //-------------------------------
		  // Descendants:
		  // Add specific Addons to fAddons
		  //-------------------------------
		}
		
    //-------------------------------------------------------
    // INETVSL methods (hence public)
    //-------------------------------------------------------

    //----------------------------------------
    public int AddonCount                        // INETVSL Exposed Interface 
    //----------------------------------------
    { get {return fAddons.Count;  }  }
		
    //----------------------------------------
    public virtual NETVSL_ResultCodes Load (Int32 InterfaceVersion)   // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // Probably no need to override
    //-------------------------------
    {
      NETVSL_ResultCodes FinalResult = NETVSL_ResultCodes.SUCCESS; 
     
      foreach (TNETVSL_Addon Addon in fAddons) {
        NETVSL_ResultCodes AResult = Addon.Load();
        if (AResult != NETVSL_ResultCodes.SUCCESS) {
          FinalResult = AResult;
          goto nomore;
        }        
      }
    nomore:
      return FinalResult;
    }

    //----------------------------------------
    public NETVSL_ResultCodes GetAddonInfo (                    // INETVSL Exposed Interface
      Int32 SessionId , 
      Int32 AddOnIx ,                                   // index starts at 1 !
      ref String Name , 
      ref NETVSL_AddOnInfo_Attribs        Attribs , 
      ref NETVSL_AddOnInfo_EnableMask     EnableMask , 
      ref NETVSL_AddOnInfo_InvokeOnMask   InvokeOnMask , 
      ref NETVSL_AddOnInfo_NotifyOnMask   NotifyOnMask ) 
    //-------------------------------
	  // Descendants:
	  // No need to override
    //-------------------------------
    {
      NETVSL_ResultCodes Result = NETVSL_ResultCodes.SUCCESS;
      TNETVSL_Addon      Addon  = AddonFromIx1(AddOnIx);
      if (Addon != null) {
        Result = Addon.GetAddonInfo(SessionId, ref Name, ref Attribs, ref EnableMask, ref InvokeOnMask, ref NotifyOnMask);
      }
      return Result;
    }

    //----------------------------------------
    public NETVSL_ResultCodes IsAddOnEnabled (System.Int32 AddOnIx , System.Int32 SessionId )  // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // No need to override
    //-------------------------------
    {
      NETVSL_ResultCodes Result = NETVSL_ResultCodes.FAILURE;
      TNETVSL_Addon      Addon  = AddonFromIx1(AddOnIx);
      if (Addon != null) {
        Result = Addon.IsAddOnEnabled(SessionId);
      }
      return Result;
    }
  
    //----------------------------------------
    public Int32 PollNextEndSessionMessage (ref Int32 SessionId )   // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // Probably no need to override
	  // Return count of remaining EndSessionIds, or -1 if we have none already
    //-------------------------------
    {  
      int Result = -1;
      SessionId  = 0;
         
      if (fEndSessionIds.Count > 0) {
        SessionId = (int) fEndSessionIds[0];
        fEndSessionIds.RemoveAt(0);
        Result = fEndSessionIds.Count;
      }
      return Result;
    }  
    
    //----------------------------------------
    public Int32 PollNextTraceMessage (ref NETVSL_AdapterMessage MsgType, ref String Text )  // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // Probably no need to override
    //-------------------------------
    {
      int Result = -1;
      MsgType    = NETVSL_AdapterMessage.None;
      Text       = "";
      
      if (fTraceMessages.Count > 0) {
        TTraceMessage ATM = (TTraceMessage) fTraceMessages[0];  
        MsgType           = ATM.MsgType;
        Text              = ATM.Msg;
        fTraceMessages.RemoveAt(0);
        Result = fTraceMessages.Count;
      }  
      return Result;
    }

    //----------------------------------------
    public virtual void KillSession ( System.Int32 SessionId )  // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // Probably no need to override
    //-------------------------------
    {
      foreach (TNETVSL_Addon Addon in fAddons) {
        Addon.KillSession(SessionId);
      }
    }

    //----------------------------------------
    public NETVSL_ResultCodes Run (                             // INETVSL Exposed Interface
    //----------------------------------------
      Int32   AddOnIx , 
      Int32   SessionId , 
      String  Args )
    //-------------------------------
	  // Descendants:
	  // Probably no need to override
    //-------------------------------
    {
      String MyArgs = Args;
      if (MyArgs == null) {
        MyArgs = "";
      }
        
      NETVSL_ResultCodes Result = NETVSL_ResultCodes.SUCCESS;
      TNETVSL_Addon      Addon  = AddonFromIx1(AddOnIx);
      if (Addon != null) {
        Result = Addon.Run(SessionId, MyArgs);
      }
      return Result;      
    }

    //----------------------------------------
    public virtual NETVSL_ResultCodes Unload (  )             // INETVSL Exposed Interface  
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // Probably no need to override
    //-------------------------------
    {
      //---------------------------------
      // Unload methods should dispose of any data
      // or other resources
      //---------------------------------
    
      foreach (TNETVSL_Addon Addon in fAddons) {
        Addon.Unload();
      }
      
      fAddons.Clear(); // actually dispose of the Addon objects themselves
      
      //------------------------------------------
      // Make sure there are no refs hanging on to Visio
      //------------------------------------------      
      if (VisApp != null)
      {
        int RefCount = 1;
        while (RefCount > 0)
        {
          RefCount = Marshal.ReleaseComObject(fVisApp);
        }
      }            
      
      return NETVSL_ResultCodes.SUCCESS;
    }

    //----------------------------------------
    public virtual Int32 Test ( Int32 I )                    // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  //   May override to perform more complicated tests
	  //-------------------------------
    {
      return I * -1;
    }

    //----------------------------------------
    public virtual void VSLInfo (ref String Descr )          // INETVSL Exposed Interface
    //----------------------------------------
    //-------------------------------
	  // Descendants:
	  // Should override to provide better string to Adapter
	  //-------------------------------
    {
      Descr = "VSLInfo default info";
    }
  }		
}
