/*-------------------------------------
A basic addon that launches a Windows Forms window

--------------------------------------*/

using System;
using NETVSL_Types;
using NETVSL_Base;
using NETVSL_Common;
using NETVSL_WinAPI;

using System.Windows.Forms;
using Visio = Microsoft.Office.Interop.Visio;

namespace NVTest_TwoAddons
{
  delegate void TFormClosing();

	//----------------------------------------
	internal class TNVTest_TwoAddons_Addon02 : TNETVSL_Addon
	//----------------------------------------
	{
	  //------------------------------------
	  // Init name and behavior options for this Addon 
	  // in the call to base constructor	
	  //------------------------------------
		public TNVTest_TwoAddons_Addon02(TNETVSL_Base AOwner):
	  //------------------------------------	  
		  base(AOwner, 
		       "NVTest_TwoAddons_Addon02"               ,
           NETVSL_AddOnInfo_Attribs.ATTS_ISACTION   , 
           NETVSL_AddOnInfo_EnableMask.ENABLEALWAYS , 
           NETVSL_AddOnInfo_InvokeOnMask.INVOKE_DEFAULT , 
           NETVSL_AddOnInfo_NotifyOnMask.NOTIFY_DEFAULT  )      
		{
      // Init data if needed
      Form1 = null; // probably redundant.
      Form2 = null;
		}
		
	  //-------------------------------------------------
	  // Run 
	  //------------------------------------------------
	  		
    //----------------------------------------
    public override NETVSL_ResultCodes Run (Int32   SessionId , String  Args ) 
    //----------------------------------------
    {
      NETVSL_ResultCodes Result = NETVSL_ResultCodes.SUCCESS;
      
      TArgHandler ArgHandler = new TArgHandler();
      
      //----------------------------------------
      // Parse Args
      //----------------------------------------      
      ArgHandler.ParseArgs(Args);            
      
      String Cmd = ArgHandler.FindValue("CMD");
      
      switch (Cmd)  {
        case "Win1_Open"  :  { Result = Form1_Open(SessionId)    ;   break;      }       
        case "Win1_Close" :  {          Form1_Close(        )    ;   break;      }        
        case "Win2_Open"  :  { Result = Form2_Open(SessionId)    ;   break;      }       
        case "Win2_Close" :  {          Form2_Close(        )    ;   break;      }        
      }      
      return Result; 
    }	
    
    //----------------------------------------
    public override NETVSL_ResultCodes Unload()
    //----------------------------------------
    {
      // return base.Unload ();  no need to call inherited
      Form1_Close();
      Form2_Close();
      return NETVSL_ResultCodes.SUCCESS;      
    }

    //-------------------------------------------------
    // Form 1: No Win32 API HWnd shenanigans
    //-------------------------------------------------
 	  private NVTest_TwoAddons_Addon02_Form1      Form1;
	  private int                                 Form1_SessionId;

    //-----------------------------
    private void Form1_Closing() {
    //-----------------------------
      Form1           = null;
      Form1_SessionId = 0;
      OwnerVSL.EnqueueEndSessionId(Form1_SessionId);      
    }
    
    //-----------------------------
    private NETVSL_ResultCodes Form1_Open(Int32 SessionId) {
    //-----------------------------
      NETVSL_ResultCodes Result = NETVSL_ResultCodes.SUCCESS;
      if (Form1 == null) {  // only open one if there isn't one already
        Form1_SessionId = SessionId;
        Form1           = new NVTest_TwoAddons_Addon02_Form1();          
        Form1.Show();
        Form1.SpecialInit(new TFormClosing(Form1_Closing), this ); // so we get notified of close
        Result = NETVSL_ResultCodes.L2V_MODELESS;
      } 
      return Result;
    }    
    //-----------------------------
    private void Form1_Close() {
    //-----------------------------
      if (Form1 != null) {  // if it's open, close it.
        Form1.Close();
      }  
    }
	
    //-------------------------------------------------
    // Form 2 -- better window management
    //-------------------------------------------------
 	  private NVTest_TwoAddons_Addon02_Form2      Form2;
	  private int                                 Form2_SessionId;

    //-----------------------------
    private void Form2_Closing() {
    //-----------------------------
      Form2           = null;
      Form2_SessionId = 0;
      OwnerVSL.EnqueueEndSessionId(Form2_SessionId);      
    }
		
    //-----------------------------
    private NETVSL_ResultCodes Form2_Open(Int32 SessionId) {
    //-----------------------------
      NETVSL_ResultCodes Result = NETVSL_ResultCodes.SUCCESS;
      if (Form2 == null) {  // only open one if there isn't one already
        Form2_SessionId = SessionId;
        Form2           = new NVTest_TwoAddons_Addon02_Form2();          
        Form2.Show();
        
        //-------------------------------------------------
        // Set window to be parented by Visio window
        //-------------------------------------------------   
        int VisHWnd  = OwnerVSL.VisApp.WindowHandle32 ;
        IntPtr FormHWnd = Form2.Handle;
        WinAPI.SetWindowLong(FormHWnd, (int) GWL.HWNDPARENT, VisHWnd);
        Form2.TopMost = true;
        
        Form2.SpecialInit(new TFormClosing(Form2_Closing), this); // see form class
        Result = NETVSL_ResultCodes.L2V_MODELESS;
      } 
      return Result;
    }    
    //-----------------------------
    private void Form2_Close() {
    //-----------------------------
      if (Form2 != null) { 
        Form2.Close();
      } 
    }    
	}
}
